package handlers

import (
	"context"
	"golias/models"
	"html/template"
	"net/http"
)

func HandleCreateRoom(w http.ResponseWriter, r *http.Request) {
	// parse payload
	payload := &models.RoomReq{
		RoomPass: r.PostFormValue("room_pass"),
		RoomName: r.PostFormValue("room_name"),
	}
	// create a room
	room, err := createRoom(r.Context(), payload)
	if err != nil {
		msg := "failed to create a room"
		log.Error(msg, "error", err)
		abortWithError(w, msg)
		return
	}
	ctx := context.WithValue(r.Context(), "current_room", room.ID)
	ctx, err = updateRoomInSession(ctx, room.ID)
	if err != nil {
		msg := "failed to set current room to session"
		log.Error(msg, "error", err)
		abortWithError(w, msg)
		return
	}
	// send msg of created room
	// h.Broker.Notifier <- broker.NotificationEvent{
	// 	EventName: models.MsgRoomListUpdate,
	// 	Payload:   fmt.Sprintf("%s created a room named %s", r.CreatorName, r.RoomName),
	// }
	// return html
	tmpl, err := template.ParseGlob("components/*.html")
	if err != nil {
		abortWithError(w, err.Error())
		return
	}
	tmpl.ExecuteTemplate(w, "base", nil)
}

func HandleRoomEnter(w http.ResponseWriter, r *http.Request) {
	// parse payload
	roomID := r.URL.Query().Get("id")
	if roomID == "" {
		// error
		return
	}
	// create a room
	room, err := getRoomByID(roomID)
	if err != nil {
		msg := "failed to find the room"
		log.Error(msg, "error", err, "room_id", roomID)
		abortWithError(w, msg)
		return
	}
	ctx := context.WithValue(r.Context(), "current_room", room.ID)
	ctx, err = updateRoomInSession(ctx, room.ID)
	if err != nil {
		msg := "failed to set current room to session"
		log.Error(msg, "error", err)
		abortWithError(w, msg)
		return
	}
	state, err := getStateByCtx(ctx)
	if err != nil {
		log.Error("failed to get state", "error", err)
		abortWithError(w, err.Error())
		return
	}
	state.RoomID = room.ID
	// update state
	if err := saveStateByCtx(ctx, state); err != nil {
		log.Error("failed to update state", "error", err)
		abortWithError(w, err.Error())
		return
	}
	// send msg of created room
	// h.Broker.Notifier <- broker.NotificationEvent{
	// 	EventName: models.MsgRoomListUpdate,
	// 	Payload:   fmt.Sprintf("%s created a room named %s", r.CreatorName, r.RoomName),
	// }
	// return html
	tmpl, err := template.ParseGlob("components/*.html")
	if err != nil {
		abortWithError(w, err.Error())
		return
	}
	tmpl.ExecuteTemplate(w, "base", room)
}

func HandleJoinTeam(w http.ResponseWriter, r *http.Request) {
	// parse payload
	// team := r.URL.Query().Get("team")
	// role := r.URL.Query().Get("role")
	r.ParseForm()
	team := r.PostFormValue("team")
	role := r.PostFormValue("role")
	if team == "" || role == "" {
		msg := "missing team or role"
		log.Error(msg)
		abortWithError(w, msg)
		return
	}
	// get username
	fi, _ := getFullInfoByCtx(r.Context())
	// get room
	if role == "mime" {
		if team == "blue" {
			fi.Room.BlueMime = fi.State.Username
			fi.State.Team = "blue"
			fi.State.Role = "mime"
		} else if team == "red" {
			fi.Room.RedMime = fi.State.Username
			fi.State.Team = "red"
			fi.State.Role = "mime"
		} else {
			msg := "uknown team:" + team
			log.Error(msg)
			abortWithError(w, msg)
			return
		}
	} else if role == "guesser" {
		if team == "blue" {
			fi.Room.BlueGuessers = append(fi.Room.BlueGuessers, fi.State.Username)
			fi.State.Team = "red"
			fi.State.Role = "guesser"
		} else if team == "red" {
			fi.Room.RedGuessers = append(fi.Room.RedGuessers, fi.State.Username)
			fi.State.Team = "red"
			fi.State.Role = "guesser"
		} else {
			msg := "uknown team:" + team
			log.Error(msg)
			abortWithError(w, msg)
			return
		}
	} else {
		msg := "uknown role:" + role
		log.Error(msg)
		abortWithError(w, msg)
		return
	}
	if err := saveFullInfoByUsername(fi.State.Username, fi); err != nil {
		log.Error("failed to save state", "error", err)
		abortWithError(w, err.Error())
		return
	}
	// return html
	tmpl, err := template.ParseGlob("components/*.html")
	if err != nil {
		abortWithError(w, err.Error())
		return
	}
	tmpl.ExecuteTemplate(w, "base", fi)
}
